@@ -3,18 +3,18 @@
 
 #define SLEPC_VERSION_RELEASE    1
 #define SLEPC_VERSION_MAJOR      3
-#define SLEPC_VERSION_MINOR      4
+#define SLEPC_VERSION_MINOR      7
 #define SLEPC_VERSION_SUBMINOR   2
 #define SLEPC_VERSION_PATCH      0
-#define SLEPC_RELEASE_DATE       "July 5, 2013"
-#define SLEPC_VERSION_DATE       "sep 02, 2013"
+#define SLEPC_RELEASE_DATE       "May 16, 2016"
+#define SLEPC_VERSION_DATE       "Jul 19, 2016"
 
 #if !defined (SLEPC_VERSION_GIT)
-#define SLEPC_VERSION_GIT        "b0629a1b2458eee7363982bb73290c27a5c787d3"
+#define SLEPC_VERSION_GIT        "v3.7.2"
 #endif
 
 #if !defined(SLEPC_VERSION_DATE_GIT)
-#define SLEPC_VERSION_DATE_GIT   "2013-09-02 11:52:34 +0200"
+#define SLEPC_VERSION_DATE_GIT   "2016-07-19 12:46:33 +0200"
 #endif
 
 #define SLEPC_VERSION_(MAJOR,MINOR,SUBMINOR) \
@@ -36,10 +36,10 @@
    SLEPC_VERSION_(MAJOR,MINOR,SUBMINOR))
 
 #define SLEPC_VERSION_GT(MAJOR,MINOR,SUBMINOR) \
-  (!SLEPC_VERSION_LE(MAJOR,MINOR,SUBMINOR))
+  (0 == SLEPC_VERSION_LE(MAJOR,MINOR,SUBMINOR))
 
 #define SLEPC_VERSION_GE(MAJOR,MINOR,SUBMINOR) \
-  (!SLEPC_VERSION_LT(MAJOR,MINOR,SUBMINOR))
+  (0 == SLEPC_VERSION_LT(MAJOR,MINOR,SUBMINOR))
 
 #endif
 
